#ifndef MIRAGE_TYPES_H
#define MIRAGE_TYPES_H

typedef unsigned char boolean;
typedef unsigned char uint8;

typedef enum
{
	MIRAGE_STATE_NORMAL,
	MIRAGE_STATE_ACTIVE,
	MIRAGE_STATE_SELECTED,
	MIRAGE_STATE_INSENSITIVE
} mirageStateType;

typedef enum
{
	MIRAGE_CORNER_NONE        = 0,
	MIRAGE_CORNER_TOPLEFT     = 1,
	MIRAGE_CORNER_TOPRIGHT    = 2,
	MIRAGE_CORNER_BOTTOMLEFT  = 4,
	MIRAGE_CORNER_BOTTOMRIGHT = 8,
	MIRAGE_CORNER_ALL         = 15
} mirageCorners;

typedef enum
{
	MIRAGE_JUNCTION_NONE      = 0,
	MIRAGE_JUNCTION_BEGIN     = 1,
	MIRAGE_JUNCTION_END       = 2
} mirageJunction;

typedef enum
{
	MIRAGE_STEPPER_UNKNOWN    = 0,
	MIRAGE_STEPPER_A          = 1,
	MIRAGE_STEPPER_B          = 2,
	MIRAGE_STEPPER_C          = 4,
	MIRAGE_STEPPER_D          = 8
} mirageStepper;

typedef enum
{
	MIRAGE_ORIENTATION_LEFT_TO_RIGHT,
	MIRAGE_ORIENTATION_RIGHT_TO_LEFT,
	MIRAGE_ORIENTATION_BOTTOM_TO_TOP,
	MIRAGE_ORIENTATION_TOP_TO_BOTTOM
} mirageOrientation;

typedef enum
{
	MIRAGE_GAP_LEFT,
	MIRAGE_GAP_RIGHT,
	MIRAGE_GAP_TOP,
	MIRAGE_GAP_BOTTOM
} mirageGapSide;

typedef enum
{
	MIRAGE_SHADOW_NONE,
	MIRAGE_SHADOW_IN,
	MIRAGE_SHADOW_OUT,
	MIRAGE_SHADOW_ETCHED_IN,
	MIRAGE_SHADOW_ETCHED_OUT
} mirageShadowType;

typedef struct
{
	double r;
	double g;
	double b;
} CairoColor;


typedef struct
{
	CairoColor bg[5];
	CairoColor base[5];
	
	CairoColor shade[8];
	CairoColor spot[3];
	
} mirageColors;

typedef struct
{
	double x;
	double y;
	double width;
	double height;
} mirageRectangle;

typedef struct
{
	boolean active;
	boolean prelight;
	boolean disabled;
	
	mirageStateType state_type;
	
	uint8 corners;
	uint8 xthickness;
	uint8 ythickness;

	CairoColor parentbg;
} WidgetParameters;

typedef struct
{
	mirageCorners    corners;
	mirageShadowType shadow;	
} ShadowParameters;

typedef struct
{
	boolean inverted;
	int     fill_size;
	boolean horizontal;
} SliderParameters;

typedef struct
{
	mirageOrientation orientation;
} ProgressBarParameters;

typedef struct
{
	int linepos;
} OptionMenuParameters;

typedef struct
{
	int separator_pos;
	boolean highlight;
} ComboBoxParameters;

typedef struct
{
	CairoColor         color;
	mirageJunction  junction;       /* On which sides the slider junctions */
	mirageStepper   steppers;       /* The visible steppers */
	boolean            horizontal;
	boolean            has_color;
} ScrollBarParameters;

typedef struct
{
	mirageStepper stepper;         /* Which stepper to draw */
} ScrollBarStepperParameters;

typedef struct
{
	mirageGapSide gap_side;
} TabParameters;

typedef struct
{
	mirageShadowType shadow;
	mirageGapSide gap_side;
	int gap_x;
	int gap_width;
	CairoColor *border;
} FrameParameters;

#define MIRAGE_RECTANGLE_SET(rect, _x, _y, _w, _h) rect.x      = _x; \
                                                      rect.y      = _y; \
                                                      rect.width  = _w; \
                                                      rect.height = _h;

#endif // MIRAGE_TYPES_H
